@@ -13,7 +13,7 @@
 #               2018/03/13 - PH Update to DPP 4.8.20
 #
 # References:   1) Bogdan private communication (Canon DPP v3.4.1.1)
-#               2) Gert Kello private communiation (DPP 3.8)
+#               2) Gert Kello private communication (DPP 3.8)
 #------------------------------------------------------------------------------
 
 package Image::ExifTool::CanonVRD;
@@ -23,7 +23,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Canon;
 
-$VERSION = '1.33';
+$VERSION = '1.35';
 
 sub ProcessCanonVRD($$;$);
 sub WriteCanonVRD($$;$);
@@ -999,7 +999,7 @@ my $blankFooter = "CANON OPTIONAL DATA\0" . ("\0" x 42) . "\xff\xd9";
     PROCESS_PROC => \&ProcessDR4,
     WRITE_PROC => \&ProcessDR4,
     WRITABLE => 1,
-    GROUPS => { 2 => 'Image' },
+    GROUPS => { 1 => 'CanonDR4', 2 => 'Image' },
     VARS => { HEX_ID => 1, SORT_PROC => \&SortDR4 },
     NOTES => q{
         Tags written by Canon DPP version 4 in CanonVRD trailers and DR4 files. Each
@@ -1276,7 +1276,7 @@ my $blankFooter = "CANON OPTIONAL DATA\0" . ("\0" x 42) . "\xff\xd9";
     WRITABLE => 1,
     FIRST_ENTRY => 0,
     FORMAT => 'int32u',
-    GROUPS => { 2 => 'Image' },
+    GROUPS => { 1 => 'CanonDR4', 2 => 'Image' },
     # 0 - value: 'IIII' (presumably byte order)
     # 1 - value: 0x00040004 (currently use this for magic number)
     # 2 - value: 6
@@ -1301,7 +1301,7 @@ my $blankFooter = "CANON OPTIONAL DATA\0" . ("\0" x 42) . "\xff\xd9";
     WRITABLE => 1,
     FIRST_ENTRY => 0,
     FORMAT => 'int32u',
-    GROUPS => { 2 => 'Image' },
+    GROUPS => { 1 => 'CanonDR4', 2 => 'Image' },
     0x00 => {
         Name => 'ToneCurveColorSpace',
         PrintConv => {
@@ -1354,7 +1354,7 @@ my $blankFooter = "CANON OPTIONAL DATA\0" . ("\0" x 42) . "\xff\xd9";
     WRITABLE => 1,
     FIRST_ENTRY => 0,
     FORMAT => 'double',
-    GROUPS => { 2 => 'Image' },
+    GROUPS => { 1 => 'CanonDR4', 2 => 'Image' },
     0x02 => 'GammaContrast',
     0x03 => 'GammaColorTone',
     0x04 => 'GammaSaturation',
@@ -1410,7 +1410,7 @@ my $blankFooter = "CANON OPTIONAL DATA\0" . ("\0" x 42) . "\xff\xd9";
     WRITABLE => 1,
     FIRST_ENTRY => 0,
     FORMAT => 'int32s',
-    GROUPS => { 2 => 'Image' },
+    GROUPS => { 1 => 'CanonDR4', 2 => 'Image' },
     0 => { Name => 'CropActive', %noYes },
     1 => 'CropRotatedOriginalWidth',
     2 => 'CropRotatedOriginalHeight',
@@ -1432,7 +1432,7 @@ my $blankFooter = "CANON OPTIONAL DATA\0" . ("\0" x 42) . "\xff\xd9";
 # DR4 Stamp Tool tags (ref PH)
 %Image::ExifTool::CanonVRD::StampInfo = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
-    GROUPS => { 2 => 'Image' },
+    GROUPS => { 1 => 'CanonDR4', 2 => 'Image' },
     FORMAT => 'int32u',
     FIRST_ENTRY => 0,
     0x02 => 'StampToolCount',
@@ -1441,7 +1441,7 @@ my $blankFooter = "CANON OPTIONAL DATA\0" . ("\0" x 42) . "\xff\xd9";
 # DR4 dust delete information (ref PH)
 %Image::ExifTool::CanonVRD::DustInfo = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
-    GROUPS => { 2 => 'Image' },
+    GROUPS => { 1 => 'CanonDR4', 2 => 'Image' },
     FORMAT => 'int32u',
     FIRST_ENTRY => 0,
     0x02 => { Name => 'DustDeleteApplied', %noYes },
@@ -2046,17 +2046,16 @@ sub ProcessCanonVRD($$;$)
             return 0;
         }
     }
-    # exit quickly if writing and no CanonVRD tags are being edited
-    if ($outfile and not exists $$et{EDIT_DIRS}{CanonVRD}) {
-        print $out "$$et{INDENT}  [nothing changed]\n" if $verbose;
-        return 1 if $outfile eq $dataPt;
-        return Write($outfile, $$dataPt) ? 1 : -1;
-    }
-
     my $vrdType = 'VRD';
 
     if ($outfile) {
         $verbose and not $created and print $out "  Rewriting CanonVRD trailer\n";
+        # exit quickly if writing and no CanonVRD tags are being edited
+        unless (exists $$et{EDIT_DIRS}{CanonVRD}) {
+            print $out "$$et{INDENT}  [nothing changed in CanonVRD]\n" if $verbose;
+            return 1 if $outfile eq $dataPt;
+            return Write($outfile, $$dataPt) ? 1 : -1;
+        }
         # delete CanonVRD information if specified
         my $doDel = $$et{DEL_GROUP}{CanonVRD};
         unless ($doDel) {
@@ -2264,7 +2263,7 @@ files, and as a trailer in JPEG, CRW, CR2 and TIFF images.
 
 =head1 AUTHOR
 
-Copyright 2003-2020, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2023, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
