@@ -16,7 +16,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.30';
+$VERSION = '1.33';
 
 sub ProcessJpeg2000Box($$$);
 sub ProcessJUMD($$$);
@@ -42,8 +42,9 @@ my %jp2Map = (
    'UUID-IPTC'   => 'JP2',
    'UUID-EXIF'   => 'JP2',
    'UUID-XMP'    => 'JP2',
-  # jp2h         => 'JP2',  (not yet functional)
-  # ICC_Profile  => 'jp2h', (not yet functional)
+    jp2h         => 'JP2',
+    colr         => 'jp2h',
+    ICC_Profile  => 'colr',
     IFD1         => 'IFD0',
     EXIF         => 'IFD0', # to write EXIF as a block
     ExifIFD      => 'IFD0',
@@ -58,7 +59,7 @@ my %jp2Map = (
 # map of where information is written in a JXL image
 my %jxlMap = (
     IFD0         => 'Exif',
-    XMP          => 'XML',
+    XMP          => 'xml ',
    'Exif'        => 'JP2',
     IFD1         => 'IFD0',
     EXIF         => 'IFD0', # to write EXIF as a block
@@ -116,9 +117,6 @@ my %j2cMarker = (
     0x76 => 'NLT', # non-linearity point transformation
 );
 
-my %jumbfTypes = (
-);
-
 # JPEG 2000 "box" (ie. atom) names
 # Note: only tags with a defined "Format" are extracted
 %Image::ExifTool::Jpeg2000::Main = (
@@ -127,10 +125,13 @@ my %jumbfTypes = (
     WRITE_PROC => \&ProcessJpeg2000Box,
     PREFERRED => 1, # always add these tags when writing
     NOTES => q{
-        The tags below are extracted from JPEG 2000 images and the JUMBF metadata in
-        JPEG images.  Note that ExifTool currently writes only EXIF, IPTC and XMP
-        tags in Jpeg2000 images.
+        The tags below are found in JPEG 2000 images and the JUMBF metadata in JPEG
+        images, but not all of these are extracted.  Note that ExifTool currently
+        writes only EXIF, IPTC and XMP tags in Jpeg2000 images.
     },
+#
+# NOTE: ONLY TAGS WITH "Format" DEFINED ARE EXTRACTED!
+#
    'jP  ' => 'JP2Signature', # (ref 1)
    "jP\x1a\x1a" => 'JP2Signature', # (ref 2)
     prfl => 'Profile',
@@ -340,10 +341,20 @@ my %jumbfTypes = (
         },
         {
             Name => 'UUID-Signature',  # (seen in JUMB data of JPEG images)
+            # (may be able to remove this when JUMBF specification is finalized)
             Condition => '$$valPt=~/^casg\x00\x11\x00\x10\x80\x00\x00\xaa\x00\x38\x9b\x71/',
             Format => 'undef',
             ValueConv => 'substr($val,16)',
         },
+        {
+            Name => 'UUID-C2PAClaimSignature',  # (seen in incorrectly-formatted JUMB data of JPEG images)
+            # (may be able to remove this when JUMBF specification is finalized)
+            Condition => '$$valPt=~/^c2cs\x00\x11\x00\x10\x80\x00\x00\xaa\x00\x38\x9b\x71/',
+            SubDirectory => {
+                TagTable => 'Image::ExifTool::CBOR::Main',
+                Start => '$valuePtr + 16',
+            },
+        },
         {
             Name => 'UUID-Unknown',
         },
@@ -386,6 +397,33 @@ my %jumbfTypes = (
         },
         SubDirectory => { TagTable => 'Image::ExifTool::JSON::Main' },
     },
+    cbor => {
+        Name => 'CBORData',
+        Flags => [ 'Binary', 'Protected' ],
+        SubDirectory => { TagTable => 'Image::ExifTool::CBOR::Main' },
+    },
+    bfdb => { # used in JUMBF (see  # (used when tag is renamed according to JUMDLabel)
+        Name => 'BinaryDataType',
+        Notes => 'JUMBF, MIME type and optional file name',
+        Format => 'undef',
+        # (ignore "toggles" byte and just extract MIME type and file name)
+        ValueConv => '$_=substr($val,1); s/\0+$//; s/\0/, /; $_',
+        JUMBF_Suffix => 'Type', # (used when tag is renamed according to JUMDLabel)
+    },
+    bidb => { # used in JUMBF
+        Name => 'BinaryData',
+        Notes => 'JUMBF',
+        Groups => { 2 => 'Preview' },
+        Format => 'undef',
+        Binary => 1,
+        JUMBF_Suffix => 'Data', # (used when tag is renamed according to JUMDLabel)
+    },
+    c2sh => { # used in JUMBF
+        Name => 'C2PASaltHash',
+        Format => 'undef',
+        ValueConv => 'unpack("H*",$val)',
+        JUMBF_Suffix => 'Salt', # (used when tag is renamed according to JUMDLabel)
+    },
 #
 # stuff seen in JPEG XL images:
 #
@@ -406,7 +444,7 @@ my %jumbfTypes = (
             ProcessProc => \&Image::ExifTool::ProcessTIFF,
             WriteProc => \&Image::ExifTool::WriteTIFF,
             DirName => 'EXIF',
-            Start => '$valuePtr + 4',
+            Start => '$valuePtr + 4 + (length($$dataPt)-$valuePtr > 4 ? unpack("N", $$dataPt) : 0)',
         },
     },
 );
@@ -529,11 +567,34 @@ my %jumbfTypes = (
 
 %Image::ExifTool::Jpeg2000::ColorSpec = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData, # (we don't actually call this)
     GROUPS => { 2 => 'Image' },
     FORMAT => 'int8s',
+    WRITABLE => 1,
+    # (Note: 'colr' is not a real group, but is used as a hack to write the
+    #  necessary colr box.  This hack necessitated another hack in TagInfoXML.pm
+    #  to avoid reporting this fake group in the XML output)
+    WRITE_GROUP => 'colr',
+    DATAMEMBER => [ 0 ],
+    IS_SUBDIR => [ 3 ],
+    NOTES => q{
+        The table below contains tags in the color specification (colr) box.  This
+        box may be rewritten by writing either ICC_Profile, ColorSpace or
+        ColorSpecData.  When writing, any existing colr boxes are replaced with the
+        newly created colr box.
+
+        B<NOTE>: Care must be taken when writing this color specification because
+        writing a specification that is incompatible with the image data may make
+        the image undisplayable.
+    },
     0 => {
         Name => 'ColorSpecMethod',
         RawConv => '$$self{ColorSpecMethod} = $val',
+        Protected => 1,
+        Notes => q{
+            default for writing is 2 when writing ICC_Profile, 1 when writing
+            ColorSpace, or 4 when writing ColorSpecData
+        },
         PrintConv => {
             1 => 'Enumerated',
             2 => 'Restricted ICC',
@@ -541,9 +602,15 @@ my %jumbfTypes = (
             4 => 'Vendor Color',
         },
     },
-    1 => 'ColorSpecPrecedence',
+    1 => {
+        Name => 'ColorSpecPrecedence',
+        Notes => 'default for writing is 0',
+        Protected => 1,
+    },
     2 => {
         Name => 'ColorSpecApproximation',
+        Notes => 'default for writing is 0',
+        Protected => 1,
         PrintConv => {
             0 => 'Not Specified',
             1 => 'Accurate',
@@ -568,6 +635,7 @@ my %jumbfTypes = (
             Name => 'ColorSpace',
             Condition => '$$self{ColorSpecMethod} == 1',
             Format => 'int32u',
+            Protected => 1,
             PrintConv => { # ref 15444-2 2002-05-15
                 0 => 'Bi-level',
                 1 => 'YCbCr(1)',
@@ -597,6 +665,8 @@ my %jumbfTypes = (
         {
             Name => 'ColorSpecData',
             Format => 'undef[$size-3]',
+            Writable => 'undef',
+            Protected => 1,
             Binary => 1,
         },
     ],
@@ -690,6 +760,7 @@ sub ProcessJUMD($$$)
         if ($len) {
             $name =~ s/[^-_a-zA-Z0-9]([a-z])/\U$1/g; # capitalize characters after illegal characters
             $name =~ tr/-_a-zA-Z0-9//dc;    # remove other illegal characters
+            $name =~ s/__/_/;               # collapse double underlines
             $name = ucfirst $name;          # capitalize first letter
             $name = "Tag$name" if length($name) < 2; # must at least 2 characters long
             $$et{JUMBFLabel} = $name;
@@ -705,7 +776,22 @@ sub ProcessJUMD($$$)
         $et->HandleTag($tagTablePtr, 'sig', substr($$dataPt, $pos, 32));
         $pos += 32;
     }
-    $pos == $end or $et->Warn('Extra data in JUMD box'." $pos $end", 1);
+    my $more = $end - $pos;
+    if ($more) {
+        # (may find c2sh box hiding after JUMD record)
+        if ($more >= 8) {
+            my %dirInfo = (
+                DataPt   => $dataPt,
+                DataLen  => $$dirInfo{DataLen},
+                DirStart => $pos,
+                DirLen   => $more,
+                DirName  => 'JUMDPrivate',
+            );
+            $et->ProcessDirectory(\%dirInfo, GetTagTable('Image::ExifTool::Jpeg2000::Main'));
+        } else {
+            $et->Warn("Extra data in JUMD box $more bytes)", 1);
+        }
+    }
     return 1;
 }
 
@@ -741,8 +827,8 @@ sub CreateNewBoxes($$)
     # add UUID boxes (and/or JXL Exif/XML boxes)
     foreach $dirName (sort keys %$addDirs) {
         # handle JPEG XL XMP and EXIF
-        if ($dirName eq 'XML' or $dirName eq 'Exif') {
-            my ($tag, $dir) = $dirName eq 'XML' ? ('xml ', 'XMP') : ('Exif', 'EXIF');
+        if ($dirName eq 'xml ' or $dirName eq 'Exif') {
+            my ($tag, $dir) = $dirName eq 'xml ' ? ('xml ', 'XMP') : ('Exif', 'EXIF');
             my $tagInfo = $Image::ExifTool::Jpeg2000::Main{$tag};
             $tagInfo = $$tagInfo[1] if ref $tagInfo eq 'ARRAY'; # (hack for stupid JXL XMP)
             my $subdir = $$tagInfo{SubDirectory};
@@ -785,6 +871,48 @@ sub CreateNewBoxes($$)
     return 1;
 }
 
+#------------------------------------------------------------------------------
+# Create Color Specification Box
+# Inputs: 0) ExifTool object ref, 1) Output file or scalar ref
+# Returns: 1 on success
+sub CreateColorSpec($$)
+{
+    my ($et, $outfile) = @_;
+    my $meth   = $et->GetNewValue('Jpeg2000:ColorSpecMethod');
+    my $prec   = $et->GetNewValue('Jpeg2000:ColorSpecPrecedence') || 0;
+    my $approx = $et->GetNewValue('Jpeg2000:ColorSpecApproximation') || 0;
+    my $icc    = $et->GetNewValue('ICC_Profile');
+    my $space  = $et->GetNewValue('Jpeg2000:ColorSpace');
+    my $cdata  = $et->GetNewValue('Jpeg2000:ColorSpecData');
+    unless ($meth) {
+        if ($icc) {
+            $meth = 2;
+        } elsif (defined $space) {
+            $meth = 1;
+        } elsif (defined $cdata) {
+            $meth = 4;
+        } else {
+            $et->Warn('Color space not defined'), return 0;
+        }
+    }
+    if ($meth eq '1') {
+        defined $space or $et->Warn('Must specify ColorSpace'), return 0;
+        $cdata = pack('N', $space);
+    } elsif ($meth eq '2' or $meth eq '3') {
+        defined $icc or $et->Warn('Must specify ICC_Profile'), return 0;
+        $cdata = $icc;
+    } elsif ($meth eq '4') {
+        defined $cdata or $et->Warn('Must specify ColorSpecData'), return 0;
+    } else {
+        $et->Warn('Unknown ColorSpecMethod'), return 0;
+    }
+    my $boxhdr = pack('N', length($cdata) + 11) . 'colr';
+    Write($outfile, $boxhdr, pack('CCC',$meth,$prec,$approx), $cdata) or return 0;
+    ++$$et{CHANGED};
+    $et->VPrint(1, "    + Jpeg2000:ColorSpec\n");
+    return 1;
+}
+
 #------------------------------------------------------------------------------
 # Process JPEG 2000 box
 # Inputs: 0) ExifTool object reference, 1) dirInfo reference, 2) Pointer to tag table
@@ -795,14 +923,14 @@ sub ProcessJpeg2000Box($$$)
     my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $dataLen = $$dirInfo{DataLen};
-    my $dataPos = $$dirInfo{DataPos};
+    my $dataPos = $$dirInfo{DataPos} || 0;
     my $dirLen = $$dirInfo{DirLen} || 0;
     my $dirStart = $$dirInfo{DirStart} || 0;
     my $base = $$dirInfo{Base} || 0;
     my $raf = $$dirInfo{RAF};
     my $outfile = $$dirInfo{OutFile};
     my $dirEnd = $dirStart + $dirLen;
-    my ($err, $outBuff, $verbose);
+    my ($err, $outBuff, $verbose, $doColour);
 
     if ($outfile) {
         unless ($raf) {
@@ -810,13 +938,19 @@ sub ProcessJpeg2000Box($$$)
             $outBuff = '';
             $outfile = \$outBuff;
         }
+        # determine if we will be writing colr box
+        if ($$dirInfo{DirName} and $$dirInfo{DirName} eq 'JP2Header') {
+            $doColour = 2 if defined $et->GetNewValue('ColorSpecMethod') or $et->GetNewValue('ICC_Profile') or
+                             defined $et->GetNewValue('ColorSpecPrecedence') or defined $et->GetNewValue('ColorSpace') or
+                             defined $et->GetNewValue('ColorSpecApproximation') or defined $et->GetNewValue('ColorSpecData');
+        }
     } else {
         # (must not set verbose flag when writing!)
         $verbose = $$et{OPTIONS}{Verbose};
         $et->VerboseDir($$dirInfo{DirName}) if $verbose;
     }
     # loop through all contained boxes
-    my ($pos, $boxLen);
+    my ($pos, $boxLen, $lastBox);
     for ($pos=$dirStart; ; $pos+=$boxLen) {
         my ($boxID, $buff, $valuePtr);
         my $hdrLen = 8;     # the box header length
@@ -825,9 +959,7 @@ sub ProcessJpeg2000Box($$$)
             my $n = $raf->Read($buff,$hdrLen);
             unless ($n == $hdrLen) {
                 $n and $err = '', last;
-                if ($outfile) {
-                    CreateNewBoxes($et, $outfile) or $err = 1;
-                }
+                CreateNewBoxes($et, $outfile) or $err = 1 if $outfile;
                 last;
             }
             $dataPt = \$buff;
@@ -839,6 +971,17 @@ sub ProcessJpeg2000Box($$$)
         }
         $boxLen = unpack("x$pos N",$$dataPt);   # (length includes header and data)
         $boxID = substr($$dataPt, $pos+4, 4);
+        # remove old colr boxes if necessary
+        if ($doColour and $boxID eq 'colr') {
+            if ($doColour == 1) { # did we successfully write the new colr box?
+                $et->VPrint(1,"    - Jpeg2000:ColorSpec\n");
+                ++$$et{CHANGED};
+                next;
+            }
+            $et->Warn('Out-of-order colr box encountered');
+            undef $doColour;
+        }
+        $lastBox = $boxID;
         $pos += $hdrLen;                # move to end of box header
         if ($boxLen == 1) {
             # box header contains an additional 8-byte integer for length
@@ -932,6 +1075,14 @@ sub ProcessJpeg2000Box($$$)
                 }
             }
         }
+        # create new tag for JUMBF data values with name corresponding to JUMBFLabel
+        if ($tagInfo and $$et{JUMBFLabel} and (not $$tagInfo{SubDirectory} or $$tagInfo{BlockExtract})) {
+            $tagInfo = { %$tagInfo, Name => $$et{JUMBFLabel} . ($$tagInfo{JUMBF_Suffix} || '') };
+            delete $$tagInfo{Description};
+            AddTagToTable($tagTablePtr, '_JUMBF_' . $$et{JUMBFLabel}, $tagInfo);
+            delete $$tagInfo{Protected}; # (must do this so -j -b returns JUMBF binary data)
+            $$tagInfo{TagID} = $boxID;
+        }
         if ($verbose) {
             $et->VerboseInfo($boxID, $tagInfo,
                 Table  => $tagTablePtr,
@@ -942,21 +1093,19 @@ sub ProcessJpeg2000Box($$$)
             );
             next unless $tagInfo;
         }
-        # create new tag for JUMBF data values with name corresponding to JUMBFLabel
-        if ($$et{JUMBFLabel} and (not $$tagInfo{SubDirectory} or $$tagInfo{BlockExtract})) {
-            $tagInfo = { %$tagInfo, Name => $$et{JUMBFLabel} };
-            AddTagToTable($tagTablePtr, '_JUMBF_' . $$et{JUMBFLabel}, $tagInfo);
-            delete $$tagInfo{Protected}; # (must do this so -j -b returns JUMBF binary data)
-            $$tagInfo{TagID} = $boxID;
-        }
         if ($$tagInfo{SubDirectory}) {
             my $subdir = $$tagInfo{SubDirectory};
             my $subdirStart = $valuePtr;
+            my $subdirLen = $boxLen;
             if (defined $$subdir{Start}) {
-                #### eval Start ($valuePtr)
+                #### eval Start ($valuePtr, $dataPt)
                 $subdirStart = eval($$subdir{Start});
+                $subdirLen -= $subdirStart - $valuePtr;
+                if ($subdirLen < 0) {
+                    $subdirStart = $valuePtr;
+                    $subdirLen = 0;
+                }
             }
-            my $subdirLen = $boxLen - ($subdirStart - $valuePtr);
             my %subdirInfo = (
                 Parent => 'JP2',
                 DataPt => $dataPt,
@@ -974,10 +1123,13 @@ sub ProcessJpeg2000Box($$$)
             my $subTable = GetTagTable($$subdir{TagTable}) || $tagTablePtr;
             if ($outfile) {
                 # remove this directory from our create list
-                delete $$et{AddJp2Dirs}{$$tagInfo{Name}};
+                delete $$et{AddJp2Dirs}{$$tagInfo{Name}};   # (eg. 'EXIF' or 'XMP')
+                delete $$et{AddJp2Dirs}{$boxID};            # (eg. 'Exif' or 'xml ')
                 my $newdir;
-                # only edit writable UUID and Exif boxes
-                if ($uuid or $boxID eq 'Exif' or ($boxID eq 'xml ' and $$et{IsJXL})) {
+                # only edit writable UUID, Exif and jp2h boxes
+                if ($uuid or $boxID eq 'Exif' or ($boxID eq 'xml ' and $$et{IsJXL}) or
+                    ($boxID eq 'jp2h' and $$et{EDIT_DIRS}{jp2h}))
+                {
                     $newdir = $et->WriteDirectory(\%subdirInfo, $subTable, $$subdir{WriteProc});
                     next if defined $newdir and not length $newdir; # next if deleting the box
                 } elsif (defined $uuid) {
@@ -989,6 +1141,11 @@ sub ProcessJpeg2000Box($$$)
                 my $boxhdr = pack('N', length($newdir) + 8 + $prefixLen) . $boxID;
                 $boxhdr .= substr($$dataPt, $valuePtr, $prefixLen) if $prefixLen;
                 Write($outfile, $boxhdr, $newdir) or $err = 1;
+                # write new colr box immediately after ihdr
+                if ($doColour and $boxID eq 'ihdr') {
+                    # (shouldn't be multiple ihdr boxes, but just in case, write only 1)
+                    $doColour = $doColour==2 ? CreateColorSpec($et, $outfile) : 0;
+                }
             } else {
                 # extract as a block if specified
                 $subdirInfo{BlockInfo} = $tagInfo if $$tagInfo{BlockExtract};
@@ -1225,7 +1382,7 @@ files.
 
 =head1 AUTHOR
 
-Copyright 2003-2021, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2023, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
