@@ -16,7 +16,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Canon;
 
-$VERSION = '1.06';
+$VERSION = '1.07';
 
 sub ProcessRealMeta($$$);
 sub ProcessRealProperties($$$);
@@ -608,7 +608,7 @@ sub ProcessReal($$)
         } else {
             last if $tag eq 'DATA'; # stop normal parsing at DATA tag
         }
-        if ($size & 0x80000000) {
+        if ($size & 0x80000000 or $size < 10) {
             $et->Warn('Bad chunk header');
             last;
         }
@@ -715,7 +715,7 @@ little-endian, but the Real format is big-endian.
 
 =head1 AUTHOR
 
-Copyright 2003-2017, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2023, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
