@@ -293,20 +293,19 @@ sub WritePDF($$)
     my $newTool = new Image::ExifTool;
     $newTool->Options(List => 1);
     $newTool->Options(Password => $et->Options('Password'));
+    $newTool->Options(NoPDFList => $et->Options('NoPDFList'));
     $$newTool{PDF_CAPTURE} = \%capture;
     my $info = $newTool->ImageInfo($raf, 'XMP', 'PDF:*', 'Error', 'Warning');
     # not a valid PDF file unless we got a version number
     # (note: can't just check $$info{PDFVersion} due to possibility of XMP-pdf:PDFVersion)
     my $vers = $newTool->GetInfo('PDF:PDFVersion');
-    ($pdfVer) = values %$vers;
+    # take highest version number if multiple versions in an incremental save
+    ($pdfVer) = sort { $b <=> $a } values %$vers;
     $pdfVer or $et->Error('Missing PDF:PDFVersion'), return 0;
     # check version number
     if ($pdfVer > 1.7) {
-        if ($pdfVer >= 2.0) {
-            $et->Error("Can't yet write PDF version $pdfVer"); # (future major version changes)
-            return 1;
-        }
-        $et->Warn("ExifTool is untested with PDF version $pdfVer files", 1);
+        $et->Warn("The PDF $pdfVer specification is not freely available", 1);
+        # (so writing by ExifTool is based on trial and error)
     }
     # fail if we had any serious errors while extracting information
     if ($capture{Error} or $$info{Error}) {
@@ -396,6 +395,10 @@ sub WritePDF($$)
 
     # must pre-determine Info reference to be used in encryption
     my $infoRef = $prevInfoRef || \ "$nextObject 0 R";
+    unless (ref $infoRef eq 'SCALAR') {
+        $et->Error("Info dictionary is not an indirect object");
+        return $rtn;
+    }
     $keyExt = $$infoRef;
 
     # must encrypt all values in dictionary if they came from an encrypted stream
@@ -751,7 +754,7 @@ C<PDF-update> pseudo group).
 
 =head1 AUTHOR
 
-Copyright 2003-2019, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2023, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
